<?php
/**
 * Classe factory gnrique qui fournit une mthode getInstance.
 * Cette mthode est capable de crer une instance de la classe appelante (en utilisant ventuellement le contructeur public et les paramtres adapts)
 * @param array $arrArgs arguments dynamiques
 * @return object instance de l'objet
 */

abstract class ploopiFactory
{
    public static function getInstance()
    {
        // Lecture des arguments
        $arrArgs = func_get_args();

        // Dtermination de la classe appelante (hrite)
        $strClassName = get_called_class();

        // Vrification d'existence (utile ?)
        if (!class_exists($strClassName)) throw new ploopiException("Unknown class &laquo; {$strClassName} &raquo;");

        // Cration d'un objet Reflexion de la classe
        $objReflection = new ReflectionClass($strClassName);

        // Vrification de l'instanciabilit de la classe appelante (hrite)
        if (!$objReflection->isInstantiable()) throw new ploopiException("Class &laquo; {$strClassName} &raquo; not instanciable");

        // Lecture du constructeur de la classe appelante (hrite)
        $objConstructMethod = $objReflection->getConstructor();

        // Vrification de l'existence d'un constructeur public (pour l'appeler)
        if ($objConstructMethod instanceof ReflectionMethod && $objConstructMethod->isPublic())
        {
            // On vrifie le nombre de paramtres attendus par le constructeur de la classe appelante (hrite)
            $intRequiredParams = $objConstructMethod->getNumberOfRequiredParameters();
            if (sizeof($arrArgs) < $intRequiredParams) throw new ploopiException("The factory method &laquo; {$strClassName}.getInstance() &raquo; requires at least {$intRequiredParams} parameter(s) : \n".implode(', ', $objMethod->getParameters()));

            return $objReflection->newInstanceArgs($arrArgs);
        }
        else return $objReflection->newInstanceArgs();
    }

    public function getClone() { return clone $this; }
}
